local draw_scale = 1 -- sets draw scale as local for the whole file
local mode = 1
local formats = love.graphics.getCanvasFormats()
local left_offset = 0
local top_offset = 0


function scaleSetup()
	
	if formats.rgba8 then
		mode = 2
	else
		mode = 1
	end
end


function scaleStart(original_width, original_height, scissor)
	if mode == 2 then
		canvasScale(original_width, original_height)
	else
		pushScale(scissor, original_width, original_height)
	end
	
	love.graphics.print("canvas supported: "..tostring(formats.rgba8), 10, 70)
end



function pushScale(scissor, original_width, original_height)
	local width = love.graphics.getWidth()
	local height = love.graphics.getHeight()
	draw_scale = height / original_height  -- gets the scale to draw
	left_offset = (width - (original_width * draw_scale)) / 2
	top_offset = (height - (original_height * draw_scale)) / 2
	
	if scissor then
		love.graphics.setScissor(left_offset, top_offset, original_width * draw_scale, original_height * draw_scale)
	end
	
	love.graphics.push()
	love.graphics.translate(left_offset, top_offset)
	love.graphics.scale(draw_scale)
	
	scaleDraw()
	
	love.graphics.pop()
	
	love.graphics.setScissor()
end




function canvasScale(original_width, original_height)
	local width = love.graphics.getWidth()
	local height = love.graphics.getHeight()
	draw_scale = height / original_height  -- gets the scale to draw

	my_canvas = love.graphics.newCanvas(original_width, original_height, "rgba8")
		
	my_canvas:setFilter("nearest")
	
	canvasResize(width, height, original_width, original_height)
	
	

	love.graphics.clear(0,0,0,0)
	my_canvas:renderTo(scaleDraw)
	love.graphics.draw(my_canvas, left_offset, top_offset, 0, draw_scale, draw_scale)
end


function canvasResize(width, height, original_width, original_height)
	xscale, yscale = width / original_width, height / original_height

	local original_aspect = width / height
	local window_aspect = original_width / original_height
	
	if window_aspect <= original_aspect then
		draw_scale = yscale
	else
		draw_scale = xscale
	end
	
	left_offset = (width - (original_width * draw_scale)) / 2
	top_offset = (height - (original_height * draw_scale)) / 2
end
	


function clickInsideBox(x, y, x1, y1, x2, y2) -- returns true if mouse clicked inside that rectangle (x, y being the mouse pos when clicked, x1,y1 first points on box, x2, y2 second points)
	if x - left_offset >= x1 * draw_scale and x - left_offset <= x2 * draw_scale and y - top_offset >= y1 * draw_scale and y - top_offset <= y2 * draw_scale then
		return true
	else 
		return false
	end
end

function scale()
	return draw_scale
end

function pointOfClick(x, y)
	x1 = x / draw_scale
	y1 = y / draw_scale
	
	return x1, y1
end